/** -------------------------------------------------------------------
  *      maintainer: Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * projects:
  *	- Jaugment II
  *       part: DirectoryService
  *	   used in release: current      used in version: current
  * -------------------------------------------------------------------
  * Copyright:   Copyright (c) 2000 by Marcus Wolschon <Suran@gmx.net>
  * -------------------------------------------------------------------
  * changes:
  * 0.0.1 (2001-03-20, Marcus Wolschon<Suran@gmx.net>)
  *       - created
  * -------------------------------------------------------------------
  * TODO:
  *       - make it work
  */
package jaugment2.directory.contexts;

  //JDK
import java.lang.Object;
import java.lang.String;
import java.util.NoSuchElementException;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.net.URL;
import java.net.HttpURLConnection;
import java.io.InputStream;
  //JNDI
import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.NamingEnumeration;
import javax.naming.spi.InitialContextFactory;
import javax.naming.NameParser;
import javax.naming.Name;
import javax.naming.Binding;
import javax.naming.directory.BasicAttributes;
import javax.naming.directory.BasicAttribute;
  //jAugment
import jaugment2.helpers.Logger;
import jaugment2.directory.FileObjectFactory;


/**
 * DictionaryContext using other contexts
 */
public class MetaDictionaryContext extends SimpleDictionaryContext
{
  private LinkedList contexts = new LinkedList();

  public MetaDictionaryContext()
  {
   LinkedList contextinfos = (LinkedList)System.getProperties().get("jaugment2.directory.contexts.MetaDictionaryContext.contexts");
   FileObjectFactory fac = new FileObjectFactory();
   for(int i=0; i<contextinfos.size(); i++)
      {
       Object o = fac.createStoredDirectory(contextinfos.get(i).toString());
       if(o != null)
       if(o instanceof Context)
          contexts.add(o);
      }
  }

  public NamingEnumeration list(String p0) throws NamingException
  {
   Logger.debug.println("DEBUG: [MetaDictionaryContext] list(p0=\""+p0+"\")");
   LinkedList l = new LinkedList();
   NamingEnumeration li= listBindings(p0);
   while(li.hasMoreElements())
     {
      Object o = li.next();
      if(o instanceof String)
        {
         l.add(o);
        }
      else
        {
         Binding b = (Binding)o;
         l.add(b.getName());
        }
     }
   return(new LinkedListEnumeration(l));
  }


  public NamingEnumeration listBindings(String p0) throws NamingException
  {
   Logger.debug.println("DEBUG: [MetaDictionaryContext] listBindings(p0=\""+p0+"\")");
   return(new BindingsEnumeration(contexts, p0));
  }



  public Object lookup(String p0) throws NamingException
  {
   Logger.debug.println("DEBUG: [MetaDictionaryContext] lookup(p0=\""+p0+"\")");
   return(new BindingsEnumeration(contexts, p0));
  }


  private class BindingsEnumeration implements NamingEnumeration
  {
   private LinkedList contexts = new LinkedList();
   private String searchfor;
   private NamingEnumeration currentEnum = null;
   public BindingsEnumeration(LinkedList contexts, String searchfor)
   {
    this.contexts  = contexts;
    this.searchfor = searchfor;
   }

   public void close() throws NamingException
   {
    if(currentEnum!=null)
      currentEnum.close();
    currentEnum = null;
   };

   private void nextEnum()
   {
    try
      {
       if(currentEnum!=null)
         currentEnum.close();
      }
    catch(Exception e)
      {}
    if(contexts.size()==0)
      {
       currentEnum = null;
       return;
      }
    try
      {
       Context c = (Context)contexts.removeFirst();
       currentEnum = c.listBindings(searchfor);
      }
    catch(Exception e)
      {
       e.printStackTrace(Logger.err);
       nextEnum();
      }
   }

   public boolean hasMore() throws NamingException
   {
    return(hasMoreElements());
   }

   public Object next() throws NamingException
   {
    return(nextElement());
   }

   public boolean hasMoreElements()
   {
    if(currentEnum==null || !currentEnum.hasMoreElements())
       nextEnum();
    return((currentEnum != null && currentEnum.hasMoreElements()));
   }

   public Object nextElement()
   {
    if(currentEnum==null || !currentEnum.hasMoreElements())
       nextEnum();

    if(currentEnum != null && currentEnum.hasMoreElements())
      return(currentEnum.nextElement());
    return(null);
   }

  }
}